@@ -188,9 +188,11 @@ shinyUI <- function(ui, path='/') {
   
   registerClient({
     
-    function(ws, header) {
+    function(req) {
+      if (!identical(req$REQUEST_METHOD, 'GET'))
+        return(NULL)
 
-      if (header$PATH != path)
+      if (req$PATH_INFO != path)
         return(NULL)
       
       textConn <- textConnection(NULL, "w") 
